#include "Day.h"

const int Day::DAYS_PER_MONTH[]
    = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

Day *Day::add_days(int n)
{
    Day *day = this;

    while (n > 0)
    {
        day = day->next_day();
        n--;
    }

    while (n < 0)
    {
        day = day->previous_day();
        n++;
    }

    return day;
}

int Day::days_from(Day *other)
{
    Day *day = this;
    int n = 0;

    while (day->compare_to(other) > 0)
    {
        day = day->previous_day();
        n++;
    }

    while (day->compare_to(other) < 0)
    {
        day = day->next_day();
        n--;
    }

    return n;
}

int Day::days_per_month(const int year, const int month)
{
   int days = DAYS_PER_MONTH[month - 1];
   if (month == FEBRUARY && is_leap_year(year)) days++;

   return days;
}

bool Day::is_leap_year(const int year)
{
   if (year%4 != 0) return false;
   if (year < GREGORIAN_START_YEAR) return true;

   return (year%100 != 0) || (year%400 == 0);
}

int Day::compare_to(const Day *other) const
{
    if (year > other->year)   return 1;
    if (year < other->year)   return -1;
    if (month > other->month) return 1;
    if (month < other->month) return -1;

    return date - other->date;
}

Day *Day::next_day() const
{
    int y = year;
    int m = month;
    int d = date;

    if (   y == GREGORIAN_START_YEAR
        && m == GREGORIAN_START_MONTH
        && d == JULIAN_END_DATE)       d = GREGORIAN_START_DATE;
    else if (d < days_per_month(y, m)) d++;
    else
    {
        d = 1;
        m++;

        if (m > DECEMBER)
        {
            m = JANUARY;
            y++;
            if (y == 0) y++;  // no year 0
        }
    }

    return new Day(y, m, d);
}

Day *Day::previous_day() const
{
    int y = year;
    int m = month;
    int d = date;

    if (   y == GREGORIAN_START_YEAR
        && m == GREGORIAN_START_MONTH
        && d == GREGORIAN_START_DATE) d = JULIAN_END_DATE;
    else if (d > 1)                   d--;
    else
    {
        m--;

        if (m < JANUARY)
        {
            m = DECEMBER;
            y--;
            if (y == 0) y--;  // no year 0
        }

        d = days_per_month(y, m);
    }

    return new Day(y, m, d);
}
